/* comp_conv.c -- Magic C++ compile output format conversion program
   (Mostly) portable public-domain implementation
   -- Copyright(C) 2003 Magicunix Infomation Technology Limited 
   This file is part of magicd.
   magicd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
	 
   For details, see the Magic C++ World-Wide-Web page, 
   `http://www.magicunix.com',
   or send a mail to the Magic C++ developers <support@magicunix.com>.
 */ 
 
 /*this file will compile to programme : comp_conv and shell_conv*/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include "info.h"
#include "conv_base.h"
#include "tools.h"
#include <getopt.h>
#define DEBUG
#define DIR_LENGTH 300
/*
###########################################################################
# Args:
# 1 	name of machine
# 2 	working directory
# 3 	makefile 
# 4 	make target
###########################################################################
*/



int main(int argc , char **argv)
{
	extern char		*optarg;
	extern int 		optind, errno;
	/*for test, read in from test.txt*/
	char szMachine[ 50 ] ;
	char szWorkdir[ DIR_LENGTH ];
	char szMakefile[ 100 ];
	char szMakeTar[ 50 ];
	char szLine[ 1024 ];
	char makeProg[100];
	/*following infomation is parsed from the compiler's output*/
	char szFilename[ 200 ];
	static char optstring[] = "m:";
	int nLinenum;   
	int optch;
	int longindex;
	char szErrorinfo[1024]; 
	char szCurWorkdir[ DIR_LENGTH ];
	char szOldWorkdir[ DIR_LENGTH ];/*in which directory to exec commands?*/ 
	char szCmdline[1024];
	int nOpt = 1;
	char *p1;
	int bLocalhost = 0;
	FILE *lfd;
	static struct option long_opts[] = {
		{"makeprog" , 0 , 0 , 'm'},
		{0 , 0 , 0 , 0}
	};
	
	
	memset(szMachine , 0 , sizeof(szMachine));
	memset(szWorkdir , 0 , sizeof(szWorkdir));
	memset(szMakefile , 0 , sizeof(szMakefile));
	memset(szMakeTar , 0 , sizeof(szMakeTar));
	memset(szLine , 0 , sizeof(szLine));
	memset(szFilename , 0 , sizeof(szFilename));
	memset(szErrorinfo , 0 , sizeof(szErrorinfo));
	memset(szCmdline , 0 , sizeof(szCmdline));	
	memset(szOldWorkdir , 0 , sizeof(szOldWorkdir));
	memset(makeProg , 0 , sizeof(makeProg))	;

	
	if ( argc < 4 )
	{
		print_usage();
		
		exit( -3 );
	}
	
	/*get arguments*/
	strcpy(szMachine , argv[ nOpt++ ] );
	strcpy(szWorkdir , argv[ nOpt++ ] );
	strcpy(szMakefile , argv[ nOpt++ ] );
	if( argc > 6)
		strcpy(szMakeTar , argv[ nOpt++ ] );
	/*get the old working directory*/
	if( getcwd(szOldWorkdir , DIR_LENGTH ) == 0 )
	{
		printf("get current working directory failed!\n");
		util_err_log("comp_conv: get current working directory failed!" , __FILE__ , __LINE__ , errno);
	}
	/*if it's a local machine*/
	if(strcmp(szMachine , "Localhost") == 0)
		bLocalhost = 1;
	/*to support DOS format , convert '\\' to '/'*/
	
	/*setup the directory to execute the command...*/
	if( chdir(szWorkdir)  == -1 )
	{
		printf("change current working directory failed!\n");
		util_err_log("comp_conv: change current working directory failed!" , __FILE__ , __LINE__ , errno);
		  
	}
	strcpy(makeProg , "make");
	while( (optch = getopt_long( argc , argv , optstring , long_opts , &longindex) ) != -1 )
	{

		switch( optch )
		{
		case 'm':  /*load arguments from file*/
			memset(makeProg , 0 , sizeof(makeProg));
			strcpy(makeProg , optarg );
			util_trim( makeProg );
			if( strlen( makeProg ) == 0 )
			{
				print_usage();
				exit( -3 );
			}
			break;
		default :
			print_usage();
			exit(-3);
		}

	}
	/*printf("Change working directory to %s\n" , szWorkdir);*/
	sprintf( szCmdline , "%s -f %s %s 2>&1" , makeProg , szMakefile , szMakeTar);
	
	if ( (lfd = popen(szCmdline,"r")) == NULL )
	{
		printf("comp_conv open pipe failed!\n");
		util_err_log("comp_conv: open pipe failed!" , __FILE__ , __LINE__ , errno);
	}
	/*
	
	
	if ( (lfd = fopen("/home/ken/magicd-2.5.0/bin/test.txt" , "r")) == NULL )
	{
		printf("comp_conv open pipe failed!\n");
		util_err_log("comp_conv: open pipe failed!" , __FILE__ , __LINE__ , errno);

	}
	*/
	/*firstly , the current working directory is top most directory*/
	strcpy(szCurWorkdir , szWorkdir  );
	while( util_getline( lfd , 1023 , szLine) != -1)
	{
		memset(szFilename , 0 , sizeof(szFilename));
		if( ( nLinenum = parseline(szLine , szFilename , szErrorinfo , lfd) ) != -1 ) /*this is a warning or error line*/
		{

			char *p1;
			/*if the first char in szErrorinfo is ':' */
			/*replace it to white space*/
			p1 = skipToNonWhite( szErrorinfo );
			if( p1 && *p1 == ':' )
				*p1 = ' ';
			if(bLocalhost == 1)
			{
				util_replace(szFilename , '/' , '\\');
				util_replace(szCurWorkdir , '/' , '\\');
			}
			/*printf("szFilename = %s\n" , szFilename)	;*/
			/*relative path name*/
			if(szFilename[ 0 ] != '/' &&  szFilename[ 1 ] != ':' && szFilename[ 0 ] != '\\' ) 
			{
				if(bLocalhost == 1)
				{
					printf("%s\\%s(%d):%s\n" , 
						szCurWorkdir , szFilename , nLinenum , szErrorinfo);
				}
				else
					
					printf("//%s%s/%s(%d):%s\n" , szMachine , 
						szCurWorkdir , szFilename , nLinenum , szErrorinfo);	
			}
#ifdef MINGW
			/*the path is cygwin path like*/
			else  if(strncasecmp(szFilename , "\\cygdrive\\" , 10 ) == 0 )
				/*cygwin form */
			{
				/*convert the string like '\cygdrive\e' to the form 'e:'*/
				
				char replaceto[200];
				memset(replaceto , 0 , sizeof(replaceto));

				replaceto[0] = szFilename[10];
				replaceto[1] = ':';
				strcat(replaceto , szFilename + 11);
				memset(szFilename , 0 ,sizeof(szFilename));
				strcpy(szFilename , replaceto);
				printf("%s(%d):%s\n" , 
						  szFilename , nLinenum , szErrorinfo);
			}

#endif
			else
			{
				if(bLocalhost == 1)
				{
					printf("%s(%d):%s\n" , 
						  szFilename , nLinenum , szErrorinfo);
				}
				else
					printf("//%s%s(%d):%s\n" , szMachine , 
						szFilename , nLinenum , szErrorinfo);		
			}
			continue;
			
		}
		/*is it changed current working directory*/
		/*just like :make[1]: Entering directory `/path/to/'*/
		/*or Leaving directory `/path/to'*/
		Checkdir (szLine , szCurWorkdir , szWorkdir ) ;
		printf("%s\n" , szLine);
	}
	if( chdir(szOldWorkdir) == -1 )
	{
		printf("restore current working directory failed!\n");
		util_err_log("comp_conv: restore current working directory failed!" , __FILE__ , __LINE__ , errno);
		
	}
	printf("Leave the directory: %s\n" , szWorkdir);
	printf("*********Make Finished*********\n");
	/*restore the directory */

}


int print_usage()
{
	printf("Usage : comp_conv mac_name work_dir make_file [make_target] [-m makeprog] \n");
	printf("this program should called by fornt-end IDE , Do not execute it by yourself!\n");
}
